-- |
-- This module exports functions specialized for the `Op` type.
module Contravariant.Extras.Op
(
  -- |
  -- A berserk collection of @contrazip@ functions with arities of up to 42.
  module Contravariant.Extras.Op.Contrazip,
)
where

import Contravariant.Extras.Op.Contrazip
